@@ -5,9 +5,9 @@
 
 ;; Author: Enami Tsugutomo <enami@ba2.so-net.or.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-isearch.el,v 1.35 2001/12/16 05:03:10 czkmt Exp $
+;; Version: $Id: skk-isearch.el,v 1.41 2002/09/01 00:57:37 czkmt Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2001/12/16 05:03:10 $
+;; Last Modified: $Date: 2002/09/01 00:57:37 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -79,7 +79,7 @@
 
 (defsubst skk-isearch-conversion-start ()
   "Point where conversion is start.  Includes skk marker."
-  (- skk-henkan-start-point skk-kanji-len))
+  (1- skk-henkan-start-point))
 
 (defsubst skk-isearch-skk-kakutei ()
   "Perform kakutei."
@@ -236,7 +236,7 @@ kakutei'ed and erase the buffer contents."
   (set skk-isearch-overriding-local-map skk-isearch-mode-map)
   ;; Input Method $B$H$7$F(B SKK $B$r;H$C$F$$$k>l9g$NBP:v(B
   (static-when
-      (memq skk-emacs-type '(mule3 mule4 mule5))
+      (memq skk-emacs-type '(mule4 mule5))
     (when (and current-input-method
 	       (string-match "^japanese-skk" current-input-method))
       (let* ((method current-input-method)
@@ -265,8 +265,7 @@ kakutei'ed and erase the buffer contents."
   (let ((mode (skk-current-input-mode)))
     (when skk-isearch-use-previous-mode
       (setq skk-isearch-mode
-	    (with-current-buffer (get-buffer-create
-				  skk-isearch-working-buffer)
+	    (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
 	      (skk-isearch-current-numerical-mode))))
     ;; reset the overrinding-local-map.
     (set skk-isearch-overriding-local-map nil)
@@ -290,9 +289,8 @@ kakutei'ed and erase the buffer contents."
        (skk-jisx0208-latin-mode-on))))
   ;; Input Method $B$H$7$F(B SKK $B$r;H$C$F$$$k>l9g$NBP:v(B
   (static-when
-      (memq skk-emacs-type '(mule3 mule4 mule5))
-    (when (string-match "^japanese-skk" (format "%s"
-						default-input-method))
+      (memq skk-emacs-type '(mule4 mule5))
+    (when (string-match "^japanese-skk" (format "%s" default-input-method))
       (with-current-buffer (get-buffer-create skk-isearch-working-buffer)
 	(inactivate-input-method))))
   ;; skk-isearch $B$N>uBV$rI=$9FbItJQ?t$N@_Dj(B
@@ -344,6 +342,9 @@ Optional argument PREFIX is appended if given."
   (define-key map "\C-g" 'skk-isearch-keyboard-quit)
   (define-key map skk-kakutei-key 'skk-isearch-newline)
   (define-key map "\C-m" 'skk-isearch-exit)
+  (when (characterp skk-previous-candidate-char)
+    (define-key map (char-to-string skk-previous-candidate-char)
+      'skk-isearch-wrapper))
 
   ;; C-x map for skk.
   (define-key map "\C-x" (make-sparse-keymap))
@@ -351,7 +352,7 @@ Optional argument PREFIX is appended if given."
 
   ;; Keys for `skk-isearch-skk-mode'.
   (let ((commands '(skk-mode skk-auto-fill-mode)))
-    (static-when (memq skk-emacs-type '(mule3 mule4 mule5))
+    (static-unless (eq skk-emacs-type '(xemacs))
       (when (string-match "^japanese-skk"
 			  (format "%s" default-input-method))
 	(push 'toggle-input-method commands)))
@@ -491,12 +492,31 @@ If the current mode is different from previous, remove it first."
 		  (setq skk-henkan-count 0)
 		  (skk-kakutei)
 		  (isearch-message))
+		 ((and (eq skk-henkan-mode 'active)
+		       skk-delete-implies-kakutei)
+		  (skk-kakutei)
+		  (delete-backward-char 1)
+		  (setq isearch-string (concat isearch-string
+					       (buffer-string))
+			isearch-message (concat
+					 (skk-isearch-mode-string)
+					 (mapconcat
+					  #'isearch-text-char-description
+					  isearch-string "")))
+		  (isearch-push-state)
+		  (isearch-update)
+		  (erase-buffer))
 		 (t
-		  (if (string= skk-prefix "")
-		      ;; now, we can't pass the universal argument within the
-		      ;; isearch-mode.  so hard code the value `1'.
-		      (delete-backward-char 1)
-		    (skk-erase-prefix 'clean))
+		  (cond
+		   ((eq skk-henkan-mode 'active)
+		    ;; In this case, `skk-delete-implies-kakutei' is nil.
+		    (skk-previous-candidate))
+		   ((string= skk-prefix "")
+		    ;; now, we can't pass the universal argument within
+		    ;; the isearch-mode.  so hard code the value `1'.
+		    (delete-backward-char 1))
+		   (t
+		    (skk-erase-prefix 'clean)))
 		  (setq skk-isearch-incomplete-message (buffer-string))
 		  (skk-isearch-incomplete-message))))))
     ;;
@@ -593,13 +613,7 @@ If the current mode is different from previous, remove it first."
 		 ((fboundp 'isearch-process-search-multibyte-characters)
 		  ;; internationalized isearch.el binds all
 		  ;; multibyte characters to `isearch-printing-char'.
-		  (cond
-		   (;; STRING is not a multibyte character.
-		    (> skk-kanji-len (length string))
-		    ;; process a special character, such as *, |, ...
-		    (command-execute command))
-		   (t
-		    (skk-isearch-process-search-string string))))
+		  (skk-isearch-process-search-string string))
 		 ;; non internationalized isearch.el
 		 (t
 		  (command-execute command)))))
@@ -620,7 +634,7 @@ If the current mode is different from previous, remove it first."
       (setq isearch-message
 	    (concat
 	     (skk-isearch-mode-string)
-	     (mapconcat 'isearch-text-char-description isearch-string "")))
+	     (mapconcat #'isearch-text-char-description isearch-string "")))
       (setq isearch-cmds (cdr isearch-cmds))
       (isearch-push-state)
       (isearch-update))
@@ -667,38 +681,34 @@ If the current mode is different from previous, remove it first."
 ;;; This advice will be enabled before skk-isearch is loaded.
 ;;;###autoload
 (defconst skk-isearch-really-early-advice
-  '(lambda ()
-     (defadvice isearch-message-prefix (around skk-isearch-ad activate)
-       (let ((current-input-method
-	      (unless (and (boundp 'skk-isearch-switch)
-			   skk-isearch-switch)
-		current-input-method)))
-	 ad-do-it))
-     (defadvice isearch-toggle-input-method (around skk-isearch-ad activate)
-       ;; Needed for calling skk-isearch via isearch-x.
-       (cond ((string-match "^japanese-skk"
-			    (format "%s" default-input-method))
-	      (let ((skk-isearch-initial-mode-when-skk-mode-disabled
-		     'latin))
-		(skk-isearch-mode-setup)
-		(skk-isearch-skk-mode)))
-	     ((null default-input-method)
-	      ad-do-it
-	      (when (string-match "^japanese-skk"
-				  (format "%s" default-input-method))
-		(let ((skk-isearch-initial-mode-when-skk-mode-disabled
-		       'latin))
-		  (skk-isearch-mode-setup))
-		(inactivate-input-method)))
-	     (t
-	      ad-do-it)))))
+  #'(lambda ()
+      (defadvice isearch-message-prefix (around skk-isearch-ad activate)
+	(let ((current-input-method
+	       (unless (and (boundp 'skk-isearch-switch)
+			    skk-isearch-switch)
+		 current-input-method)))
+	  ad-do-it))
+      (defadvice isearch-toggle-input-method (around skk-isearch-ad activate)
+	;; Needed for calling skk-isearch via isearch-x.
+	(cond ((string-match "^japanese-skk"
+			     (format "%s" default-input-method))
+	       (let ((skk-isearch-initial-mode-when-skk-mode-disabled
+		      'latin))
+		 (skk-isearch-mode-setup)
+		 (skk-isearch-skk-mode)))
+	      ((null default-input-method)
+	       ad-do-it
+	       (when (string-match "^japanese-skk"
+				   (format "%s" default-input-method))
+		 (let ((skk-isearch-initial-mode-when-skk-mode-disabled
+			'latin))
+		   (skk-isearch-mode-setup))
+		 (inactivate-input-method)))
+	      (t
+	       ad-do-it)))))
 
 ;;;###autoload
-(cond
- ((or (featurep 'xemacs)
-      (not (fboundp 'register-input-method)))
-  nil)
- (t
+(unless (featurep 'xemacs)
   (define-key isearch-mode-map [(control \\)] 'isearch-toggle-input-method)
   (cond
    ((and (featurep 'advice)
@@ -712,7 +722,7 @@ If the current mode is different from previous, remove it first."
     (funcall skk-isearch-really-early-advice))
    (t
     ;; Emacs 21 loads "leim-list" files before `load-path' is prepared.
-    (add-hook 'before-init-hook skk-isearch-really-early-advice)))))
+    (add-hook 'before-init-hook skk-isearch-really-early-advice))))
 
 (put 'skk-isearch-wrapper 'isearch-command t)
 (put 'skk-isearch-keyboard-quit 'isearch-command t)
