@@ -1,14 +1,14 @@
 ;;; skk-num.el --- $B?tCMJQ49$N$?$a$N%W%m%0%i%`(B
 
 ;; Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-;;               1998, 1999, 2000, 2001
+;;               1998, 1999, 2000, 2001, 2002
 ;;   Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-num.el,v 1.32 2001/12/16 05:03:10 czkmt Exp $
+;; Version: $Id: skk-num.el,v 1.37 2002/04/05 02:53:40 obata Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2001/12/16 05:03:10 $
+;; Last Modified: $Date: 2002/04/05 02:53:40 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -43,12 +43,12 @@
 
 ;;;###autoload
 (defun skk-num-compute-henkan-key (key)
-  ;; KEY $B$NCf$NO"B3$9$k?t;z$r8=$o$9J8;zNs$r(B "#" $B$KCV$-49$($?J8;zNs$rJV$9!#(B"12"
-  ;; $B$d(B "$B#0#9(B" $B$J$IO"B3$9$k?t;z$r(B 1 $B$D$N(B "#" $B$KCV$-49$($k$3$H$KCm0U!#(B
-  ;; $BCV$-49$($??t;z$r(B skk-num-list $B$NCf$K%j%9%H$N7A$GJ]B8$9$k!#(B
-  ;; $BNc$($P!"(BKEY $B$,(B "$B$X$$$;$$(B7$BG/(B12$B$,$D(B" $B$G$"$l$P!"(B"$B$X$$$;$$(B#$B$M$s(B#$B$,$D(B"
-  ;; $B$HJQ49$7!"(Bskk-num-list $B$K(B ("7" "12") $B$H$$$&%j%9%H$rBeF~$9$k!#(B
-  ;; $B<-=q$N8+=P$78l$N8!:w$K;HMQ$9$k!#(B
+  "KEY $B$NCf$NO"B3$9$k?t;z$r8=$o$9J8;zNs$r(B \"#\" $B$KCV$-49$($?J8;zNs$rJV$9!#(B
+\"12\" $B$d(B \"$B#0#9(B\" $B$J$IO"B3$9$k?t;z$r(B 1 $B$D$N(B \"#\" $B$KCV$-49$($k$3$H$KCm0U!#(B
+$BCV$-49$($??t;z$r(B `skk-num-list' $B$NCf$K%j%9%H$N7A$GJ]B8$9$k!#(B
+$BNc$($P!"(BKEY $B$,(B \"$B$X$$$;$$(B7$BG/(B12$B$,$D(B\" $B$G$"$l$P!"(B\"$B$X$$$;$$(B#$B$M$s(B#$B$,$D(B\"
+$B$HJQ49$7!"(B`skk-num-list' $B$K(B (\"7\" \"12\") $B$H$$$&%j%9%H$rBeF~$9$k!#(B
+$B<-=q$N8+=P$78l$N8!:w$K;HMQ$9$k!#(B"
   (let ((numexp (if skk-num-convert-float
 		    "[.0-9]+"
 		  "[0-9]+")))
@@ -76,12 +76,12 @@
   key)
 
 ;;;###autoload
-(defun skk-num-convert ()
-  ;; skk-henkan-list $B$N(B skk-henkan-count $B$,;X$7$F$$$k8uJd(B ($B?tCMJQ49(B
-  ;; $B%-!<$N(B) $B$rJQ49$7!"(Bskk-henkan-list $B$r(B
-  ;;   ("#2" ...) -> (("#2" ."$B0l(B") ...)
-  ;; $B$N$h$&$KJQ7A$9$k!#(B
-  (let ((key (skk-get-current-candidate-1))
+(defun skk-num-convert (index)
+  "INDEX $B$,;X$9(B `skk-henkan-list' $B$NMWAG$r?tCMJQ49$N$?$a$K2C9)$9$k!#(B
+`skk-henkan-list' $B$N(B INDEX $B$,;X$7$F$$$k8uJd(B \($B?tCMJQ49%-!<$N(B)\ $B$r(B
+  \"#2\" -> \(\"#2\" .\"$B0l(B\"\)
+$B$N$h$&$KJQ49$9$k!#(B"
+  (let ((key (skk-get-current-candidate-1 index))
 	convlist current)
     (unless (consp key)
       (setq convlist (skk-num-convert-1 key))
@@ -93,7 +93,7 @@
 	(setq current (mapconcat 'identity convlist ""))
 	(if (skk-get-current-candidate-1)
 	    ;; ("A" "#2" "C") -> ("A" ("#2" ."$B0l(B") "C")
-	    (setcar (nthcdr skk-henkan-count skk-henkan-list)
+	    (setcar (nthcdr index skk-henkan-list)
 		    (cons key current))
 	  (setq skk-henkan-list
 		(nconc skk-henkan-list (list (cons key current))))))
@@ -102,22 +102,21 @@
 	(let ((l (mapcar (function (lambda (e) (cons key e)))
 			 (skk-num-flatten-list convlist))))
 	  (setq current (cdr (car l)))
-	  (if (and (> skk-henkan-count -1)
-		   (nth skk-henkan-count skk-henkan-list))
+	  (if (and (> index -1)
+		   (nth index skk-henkan-list))
 	      (progn
-		(setcar (nthcdr skk-henkan-count skk-henkan-list)
-			(car l))
+		(setcar (nthcdr index skk-henkan-list) (car l))
 		(setq skk-henkan-list (skk-splice-in
 				       skk-henkan-list
-				       (1+ skk-henkan-count)
+				       (1+ index)
 				       (cdr l))))
 	    (setq skk-henkan-list (nconc skk-henkan-list l))
 	    (skk-num-uniq))))))))
 
 (defun skk-num-convert-1 (key)
-  ;; KEY $B$r(B skk-num-list $B$K=>$$JQ49$7!"JQ498e$NJ8;zNs$N%Q!<%D$r(B
-  ;; $B=g$K$J$i$Y$?%j%9%H$rJV$9!#(B
-  ;; KEY ::= `$BJ?@.(B#0$BG/(B', return ::= ("$BJ?@.(B" "13" "$BG/(B")
+  "KEY $B$r(B `skk-num-list' $B$K=>$$JQ49$9$k!#(B
+$BJQ498e$NJ8;zNs$N%Q!<%D$r=g$K$J$i$Y$?%j%9%H$rJV$9!#Nc$($P(B
+  KEY ::= `$BJ?@.(B#0$BG/(B', return ::= (\"$BJ?@.(B\" \"13\" \"$BG/(B\")"
   (unless (or (not key)
 	      (consp key))
     (let ((numexp (if skk-num-convert-float
@@ -147,14 +146,16 @@
 
 ;;;###autoload
 (defun skk-num-multiple-convert (&optional count)
-  (let ((skk-henkan-count skk-henkan-count)
-	(n (or count (length skk-henkan-list))))
-    (while (and (> n 0) (nth skk-henkan-count skk-henkan-list))
-      (skk-num-convert)
-      ;; skk-henkan-count $B$rA`:n$7$J$/$H$b(B skk-num-convert $B$,(B
-      ;; $BM-8z$K$J$k$h$&$K$7$?$$!#(B
-      (setq skk-henkan-count (1+ skk-henkan-count)
-	    n (1- n)))))
+  (let ((index skk-henkan-count))
+    (catch 'break
+      (while (nth index skk-henkan-list)
+	(when (and count (> 0 count))
+	  (throw 'break nil))
+	(skk-num-convert index)
+	(setq index (1+ index))
+	(when count
+	  (setq count (1- count)))))
+    (skk-num-uniq)))
 
 (defun skk-num-rawnum-exp (string)
   (setq string (skk-num-rawnum-exp-1
@@ -164,7 +165,8 @@
   (setq string (skk-num-rawnum-exp-1
 		string "[$B#0(B-$B#9(B]+" "#1" 0))
   (setq string (skk-num-rawnum-exp-1
-		string "\\([$B!;0l6e8^;0;M<7FsH,O;==(B][$B==I4@iK|2/C{5~(B]\\)+" "#3" 0))
+		string "\\([$B!;0l6e8^;0;M<7FsH,O;==(B][$B==I4@iK|2/C{5~(B]\\)+"
+		"#3" 0))
   ;; (mapcar 'char-to-string
   ;;         (sort
   ;;          '(?$B0l(B ?$BFs(B ?$B;0(B ?$B;M(B ?$B8^(B ?$BO;(B ?$B<7(B ?$BH,(B ?$B6e(B ?$B!;(B) '<))
@@ -182,24 +184,26 @@
     string))
 
 (defun skk-num-flatten-list (list)
-  ;; $BM?$($i$l$?%j%9%H$N3FMWAG$+$iAH$_9g$;2DG=$JJ8;zNs$NO"@\$r:n$j!"%j%9%H$GJV(B
-  ;; $B$9!#(B
-  ;; (("A" "B") "1" ("X" "Y")) -> ("A1X" "A1Y" "B1X" "B1Y")
+  "$BM?$($i$l$?%j%9%H$N3FMWAG$+$iAH$_9g$;2DG=$JJ8;zNs$NO"@\$r:n$k!#(B
+$B7k2L$O%j%9%H$GJV$9!#Nc$($P(B
+  ((\"A\" \"B\") \"1\" (\"X\" \"Y\")) -> (\"A1X\" \"A1Y\" \"B1X\" \"B1Y\")"
   (let ((dst (car list))
 	(src (cdr list))
 	elt)
+    (unless (listp dst)
+      (setq dst (list dst)))
     (while src
       (setq elt (car src))
       (setq dst
 	    (cond
 	     ((consp elt)
-	      (apply (function nconc)
+	      (apply #'nconc
 		     (mapcar
-		      (lambda (str0)
-			(mapcar
-			 (lambda (str1)
-			   (concat str0 str1))
-			 elt))
+		      #'(lambda (str0)
+			  (mapcar
+			   #'(lambda (str1)
+			       (concat str0 str1))
+			   elt))
 		      dst)))
 	     (t
 	      (mapcar
@@ -211,23 +215,23 @@
 
 ;;;###autoload
 (defun skk-num-exp (num type)
-  ;; ascii $B?t;z(B (string) $B$N(B NUM $B$r(B TYPE $B$K=>$$JQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
-  ;; TYPE $B$O2<5-$NDL$j!#(B
-  ;; 0 -> $BL5JQ49(B
-  ;; 1 -> $BA43Q?t;z$XJQ49(B
-  ;; 2 -> $B4A?t;z$XJQ49(B ($B0L<h$j$J$7(B)
-  ;; 3 -> $B4A?t;z$XJQ49(B ($B0L<h$j$r$9$k(B)
-  ;; 4 -> $B$=$N?t;z$=$N$b$N$r%-!<$K$7$F<-=q$r:F8!:w(B
-  ;; 5 -> $B4A?t;z(B ($B<j7A$J$I$G;HMQ$9$kJ8;z$r;HMQ(B) $B$XJQ49(B ($B0L<h$j$r$9$k(B)
-  ;; 9 -> $B>-4}$G;HMQ$9$k?t;z(B ("$B#3;M(B" $B$J$I(B) $B$KJQ49(B
+  "ascii $B?t;z(B (string) $B$N(B NUM $B$r(B TYPE $B$K=>$$JQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
+TYPE $B$O2<5-$NDL$j!#(B
+0 -> $BL5JQ49(B
+1 -> $BA43Q?t;z$XJQ49(B
+2 -> $B4A?t;z$XJQ49(B ($B0L<h$j$J$7(B)
+3 -> $B4A?t;z$XJQ49(B ($B0L<h$j$r$9$k(B)
+4 -> $B$=$N?t;z$=$N$b$N$r%-!<$K$7$F<-=q$r:F8!:w(B
+5 -> $B4A?t;z(B ($B<j7A$J$I$G;HMQ$9$kJ8;z$r;HMQ(B) $B$XJQ49(B ($B0L<h$j$r$9$k(B)
+9 -> $B>-4}$G;HMQ$9$k?t;z(B (\"$B#3;M(B\" $B$J$I(B) $B$KJQ49(B"
   (save-match-data
     (let ((fun (cdr (assq type skk-num-type-alist))))
       (when fun
 	(funcall fun num)))))
 
 (defun skk-num-jisx0208-latin (num)
-  ;; ascii $B?t;z$N(B NUM $B$rA43Q?t;z$NJ8;zNs$KJQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
-  ;; $BNc$($P(B "45" $B$r(B "$B#4#5(B" $B$KJQ49$9$k!#(B
+  "ascii $B?t;z$N(B NUM $B$rA43Q?t;z$NJ8;zNs$KJQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
+$BNc$($P(B \"45\" $B$r(B \"$B#4#5(B\" $B$KJQ49$9$k!#(B"
   (let ((candidate
 	 (mapconcat (function
 		     (lambda (c)
@@ -237,8 +241,8 @@
       candidate)))
 
 (defun skk-num-type2-kanji (num)
-  ;; ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
-  ;; $BNc$($P!"(B"45" $B$r(B "$B;M8^(B" $B$KJQ49$9$k!#(B
+  "ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7!"JQ498e$NJ8;zNs$rJV$9!#(B
+$BNc$($P!"(B\"45\" $B$r(B \"$B;M8^(B\" $B$KJQ49$9$k!#(B"
   (save-match-data
     (when (skk-num-int-p num)
       (let ((candidate
@@ -252,23 +256,23 @@
 	  candidate)))))
 
 (defun skk-num-type3-kanji (num)
-  ;; ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7(B ($B0L<h$j$r$9$k(B)$B!"JQ498e$NJ8;zNs$r(B
-  ;; $BJV$9!#Nc$($P(B "1021" $B$r(B "$B@iFs==0l(B" $B$KJQ49$9$k!#(B
+  "ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7(B ($B0L<h$j$r$9$k(B)$B!"JQ498e$NJ8;zNs$r(B
+$BJV$9!#Nc$($P(B \"1021\" $B$r(B \"$B@iFs==0l(B\" $B$KJQ49$9$k!#(B"
   (save-match-data
     (when (skk-num-int-p num)
       ;; $B>.?tE@$r4^$^$J$$?t(B
       (skk-num-to-kanji num 'type3))))
 
 (defun skk-num-type5-kanji (num)
-  ;; ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7(B ($B0L<h$j$r$9$k(B)$B!"JQ498e$NJ8;zNs$r(B
-  ;; $BJV$9!#Nc$($P(B "1021" $B$r(B "$B0motFu=&0m(B" $B$KJQ49$9$k!#(B
+  "ascii $B?t;z(B NUM $B$r4A?t;z$NJ8;zNs$KJQ49$7(B ($B0L<h$j$r$9$k(B)$B!"JQ498e$NJ8;zNs$r(B
+$BJV$9!#Nc$($P(B \"1021\" $B$r(B \"$B0motFu=&0m(B\" $B$KJQ49$9$k!#(B"
   (save-match-data
     (when (skk-num-int-p num)
       ;; $B>.?tE@$r4^$^$J$$?t(B
       (skk-num-to-kanji num 'type5))))
 
 (defun skk-num-to-kanji (num type &optional alist)
-  ;; NUM $B$r(B TYPE $B$N7A<0$N4A?t;z$K$9$k!#0L$J$I$rI=$94A;z$O(B ALIST $B$+$i<hF@$9$k!#(B
+  "NUM $B$r(B TYPE $B$N7A<0$N4A?t;z$K$9$k!#0L$J$I$rI=$94A;z$O(B ALIST $B$+$i<hF@$9$k!#(B"
   (let ((len (length num))
 	(i 0)
 	char v num1 v1)
@@ -345,8 +349,8 @@
 	(skk-num-get-suuji ?0 alist))))
 
 (defun skk-num-shogi (num)
-  ;; ascii $B?t;z$N(B NUM $B$r>-4}$G;HMQ$5$l$k?t;zI=5-$KJQ49$9$k!#(B
-  ;; $BNc$($P(B "34" $B$r(B "$B#3;M(B" $B$KJQ49$9$k!#(B
+  "ascii $B?t;z$N(B NUM $B$r>-4}$G;HMQ$5$l$k?t;zI=5-$KJQ49$9$k!#(B
+$BNc$($P(B \"34\" $B$r(B \"$B#3;M(B\" $B$KJQ49$9$k!#(B"
   (save-match-data
     (when (and (= (length num) 2)
 	       (skk-num-int-p num))
@@ -357,7 +361,7 @@
 	  candidate)))))
 
 (defun skk-num-recompute (num)
-  ;; #4 $B$N8+=P$7$KBP$7!"(Bskk-henkan-key $B$KBeF~$5$l$??t;z$=$N$b$N$r:FEY8!:w$9$k!#(B
+  "#4 $B$N8+=P$7$KBP$7(B `skk-henkan-key' $B$KBeF~$5$l$??t;z$=$N$b$N$r:FEY8!:w$9$k!#(B"
   (let (result)
     (setq skk-num-recompute-key num)
     (with-temp-buffer
@@ -442,7 +446,7 @@
 
 ;;;###autoload
 (defun skk-num-initialize ()
-  ;; skk-use-numeric-convert $B4XO"$NJQ?t$r=i4|2=$9$k!#(B
+  "`skk-use-numeric-conversion' $B4XO"$NJQ?t$r=i4|2=$9$k!#(B"
   (setq skk-last-henkan-data
 	(put-alist 'num-list skk-num-list skk-last-henkan-data)
 	skk-num-list nil
@@ -450,14 +454,15 @@
 
 ;;;###autoload
 (defun skk-num-henkan-key ()
-  ;; type4 $B$N?tCM:FJQ49$,9T$J$o$l$?$H$-$O!"?tCM<+?H$rJV$7!"$=$l0J30$N?tCMJQ49(B
-  ;; $B$G$O!"(Bskk-henkan-key $B$rJV$9!#(B
+  "$BE,@Z$JJQ49%-!<$rJV$9!#(B
+type4 $B$N?tCM:FJQ49$,9T$J$o$l$?$H$-$O!"?tCM<+?H$rJV$7!"$=$l0J30$N?tCMJQ49(B
+$B$G$O!"(B`skk-henkan-key' $B$N?tCM$r(B \"#\" $B$GCV$-49$($?%-!<$rJV$9!#(B"
   (or skk-num-recompute-key
-      skk-henkan-key))
+      (skk-num-compute-henkan-key skk-henkan-key)))
 
 ;;;###autoload
 (defun skk-num-update-jisyo (noconvword word &optional purge)
-  ;; $B?t;z<+?H$r8+=P$78l$H$7$F<-=q$N%"%C%W%G!<%H$r9T$J$&!#(B
+  "$B?t;z<+?H$r8+=P$78l$H$7$F<-=q$N%"%C%W%G!<%H$r9T$J$&!#(B"
   (when (and skk-num-recompute-key
 	     (save-match-data (string-match "#4" noconvword)))
     (with-current-buffer (skk-get-jisyo-buffer skk-jisyo 'nomsg)
@@ -468,8 +473,8 @@
 
 ;;;###autoload
 (defun skk-num (str)
-  ;; $B?t;z$r(B skk-number-style $B$NCM$K=>$$JQ49$9$k!#(B
-  ;; skk-current-date $B$N%5%V%k!<%A%s!#(B
+  "$B?t;z$r(B `skk-number-style' $B$NCM$K=>$$JQ49$9$k!#(B
+`skk-current-date' $B$N%5%V%k!<%A%s!#(B"
   (mapconcat (function
 	      (lambda (c)
 		(cond
